# Cache version of call_function

#define FROM_ASM
#include "rtos.h"

#define MSR_CACHES 0xa0

.global call_function
.global init_asm_start
.global kernel_entry
.global return_point
.global region_change

.section	.kernelstuff.t,"ax",@progbits

call_function:
        # r5: address of target function
        # r6: block info word
        # r7-r10: parameters of target function
        # r15: return PC - 8

        # SYSCALL mode: disable interrupts, breaks
        msrclr  r12, MSR_IE | MSR_BE
        
        swi     r15, r17, 0     # Save my return address 
        swi     r12, r17, 8     # and entry MSR state
        addi    r17, r17, 12

        or      r4, r5, r0

        # USER mode: enable interrupts, breaks
        msrset  r3, MSR_IE | MSR_BE

        # Parameter registers moved into place
        or      r5, r7, r0
        or      r6, r8, r0
        or      r7, r9, r0

        # Call function
        brald   r15, r4
        or      r8, r10, r0

return_point:
        # SYSCALL mode: disable interrupts, breaks
        msrclr  r5, MSR_IE | MSR_BE

        addi    r17, r17, -12
        lwi     r15, r17, 0         # Return point restored
        lwi     r12, r17, 8         # and entry MSR state

        rtsd    r15, 8
        mts     rmsr, r12

region_change:
        # r15: next region start (when rounded up)
        addi    r15, r15, BLOCK_SIZE
        andni   r15, r15, BLOCK_SIZE-1  # next region start
        bra     r15

.text
kernel_entry:
        braid   init_asm_start
        addi    r3, r0, MSR_BE | MSR_CACHES

