
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library unimacro;
use unimacro.vcomponents.all;


entity stack is
port (
    clk         : in std_logic;
    rst         : in std_logic;
    push        : in std_logic;
    pop         : in std_logic;
    db_ptr      : out std_logic_vector(31 downto 0);
    topout      : out std_logic_vector(31 downto 0);
    topin       : in std_logic_vector(31 downto 0));
end entity stack;


architecture basic of stack is
    constant STACK_A_WIDTH : Natural := 13; -- 8192 words in stack
    constant BRAM_LEFT  : Natural := 9; -- 1024 words per bram
    constant NUM_BRAMS  : Natural := 8; -- 8192/1024 = 8 brams
    constant STACK_LEFT : Natural := STACK_A_WIDTH - 1;
    signal tosptr       : std_logic_vector(STACK_LEFT downto 0);
    signal ain          : std_logic_vector(STACK_LEFT downto 0);
    signal we           : std_logic_vector(0 downto 0);
begin
    process (clk, rst) is
    begin
        if (rst = '1') then
            tosptr <= (others => '0');

        elsif (clk'event and clk = '1') then
            if (push = '1') then
                tosptr <= tosptr + 1;
            elsif (pop = '1') then
                tosptr <= tosptr - 1;
            end if;
        end if;
    end process;

    process (tosptr) is
    begin
        db_ptr <= (others => '0');
        db_ptr(STACK_LEFT downto 0) <= tosptr(STACK_LEFT downto 0);
    end process;

    we <= (others => push);
    ain <= tosptr when push = '1' else (tosptr - 1);

    bram_gen : for bram in 0 to (NUM_BRAMS - 1)
    generate
        b : block
            constant right  : Natural := 4 * bram;
            constant left   : Natural := right + 3;
        begin
            stack_store : BRAM_SINGLE_MACRO
                generic map (
                    BRAM_SIZE => "36Kb",
                    WRITE_WIDTH => 4,
                    READ_WIDTH => 4)
                port map (
                    di => topin(left downto right),
                    addr => ain,
                    we => we,
                    en => '1',
                    regce => '0',
                    rst => '0',
                    clk => clk,
                    do => topout(left downto right));
        end block b;
    end generate bram_gen;

end architecture basic;


